//
//  AKLinkedList.m
//  LinkedList
//
//  Created by Aleksey Kuznetsov on 13.08.13.
//  Copyright (c) 2013 zummenix. All rights reserved.
//


#import "AKLinkedList.h"
#import "AKLinkedListNode.h"


@interface AKLinkedList ()
{
    AKLinkedListNode *_currentNode;
    AKLinkedListNode *_firstNode;
    AKLinkedListNode *_lastNode;
}

@end


@implementation AKLinkedList

- (void)addObject:(id)object
{
    if (object != nil) {
        AKLinkedListNode *node = [[AKLinkedListNode alloc] initWithObject:object];
        if ([self count] == 0) {
            _firstNode = node;
            _lastNode = node;
        } else {
            _lastNode.nextNode = node;
            node.previousNode = _lastNode;
            _lastNode = node;
        }
        _count++;
    }
}

- (BOOL)containsObject:(id)object
{
    for (id obj in self) {
        if ([object isEqual:obj]) {
            return YES;
        }
    }
    return NO;
}


#pragma mark - NSFastEnumeration

// This is where all the magic happens.
// You have two choices when implementing this method:
// 1) Use the stack based array provided by stackbuf. If you do this, then you must respect the value of 'len'.
// 2) Return your own array of objects. If you do this, return the full length of the array returned until you run out of objects, then return 0. For example, a linked-array implementation may return each array in order until you iterate through all arrays.
// In either case, state->itemsPtr MUST be a valid array (non-nil). This sample takes approach #1, using stackbuf to store results.
-(NSUInteger)countByEnumeratingWithState:(NSFastEnumerationState *)state
                                 objects:(__unsafe_unretained id *)stackbuf
                                   count:(NSUInteger)len
{
	NSUInteger count = 0;
	// This is the initialization condition, so we'll do one-time setup here.
	// Ensure that you never set state->state back to 0, or use another method to detect initialization
	// (such as using one of the values of state->extra).
	if(state->state == 0) {
        _currentNode = _firstNode;
		// We are not tracking mutations, so we'll set state->mutationsPtr to point into one of our extra values,
		// since these values are not otherwise used by the protocol.
		// If your class was mutable, you may choose to use an internal variable that is updated when the class is mutated.
		// state->mutationsPtr MUST NOT be NULL.
		state->mutationsPtr = &state->extra[0];
	}
    
	// Now we provide items, which we track with state->state, and determine if we have finished iterating.
	if(state->state < self.count) {
		// Set state->itemsPtr to the provided buffer.
		// Alternate implementations may set state->itemsPtr to an internal C array of objects.
		// state->itemsPtr MUST NOT be NULL.
		state->itemsPtr = stackbuf;
		// Fill in the stack array, either until we've provided all items from the list
		// or until we've provided as many items as the stack based buffer will hold.
		while((state->state < self.count) && (count < len)) {
            stackbuf[count] = _currentNode.object;
			state->state++;
			count++;
            _currentNode = _currentNode.nextNode;
		}
	} else {
        _currentNode = nil;
		// We've already provided all our items, so we signal we are done by returning 0.
		count = 0;
	}
	return count;
}

@end
